from .generic_diff_reporter import GenericDiffReporter, create_config


class ReportWithVSCode(GenericDiffReporter):
    def __init__(self):
        super().__init__(config=create_config(["ReportWithVSCode", "code", ["-d"]]))



import shutil

from approvaltests.core import Reporter


class ReporterThatAutomaticallyApproves(Reporter):
    def report(self, received_path: str, approved_path: str) -> bool:
        shutil.move(received_path, approved_path)
        return True



from pathlib import Path

from approval_utilities.utils import ensure_file_exists
from approvaltests.core.reporter import Reporter
from approvaltests.reporters.python_native_reporter import calculate_diff


class ReportByCreatingDiffFile(Reporter):
    def report(self, received_path: str, approved_path: str) -> bool:
        ensure_file_exists(approved_path)
        diff = calculate_diff(received_path, approved_path)
        with open(
            self.get_diff_file_name(received_path), mode="w", encoding="utf8"
        ) as file:
            file.write(diff)
        return True

    @staticmethod
    def get_diff_file_name(received_path):
        suffix = Path(received_path).suffix
        return received_path.replace(f".received{suffix}", ".diff")



from typing import Optional, Any

from approvaltests.core import Reporter

from approval_utilities.utils import write_to_temporary_file
from approvaltests.core.options import Options
from approvaltests.approvals import get_default_namer, initialize_options
from approvaltests.approvals import verify_with_namer
from approvaltests.namer.stack_frame_namer import StackFrameNamer
from approvaltests.reporters.default_reporter_factory import get_reporter
from approvaltests.reporters.testing_reporter import ReporterForTesting


class FilePathNamer(StackFrameNamer):
    def __init__(self, file_path: str, extension: None = None) -> None:
        StackFrameNamer.__init__(self, extension)
        self.file_path = file_path

    def get_approved_filename(self, basename: Optional[str] = None) -> str:
        return self.file_path


def assert_against_file(
    actual: str, file_path: str, reporter: Optional[ReporterForTesting] = None
) -> None:
    namer = FilePathNamer(file_path)
    verify_with_namer(actual, namer, reporter)


def assert_equal_with_reporter(expected:str,
                               actual:Any,
                               reporter:Reporter=None,
                               *,  # enforce keyword arguments - https://www.python.org/dev/peps/pep-3102/
                               options: Optional[Options] = None
    )->None:
    actual = options.scrub(actual)
    if actual == expected:
        return
    options = initialize_options(options, reporter)

    name = get_default_namer().get_file_name()
    extention = options.for_file.file_extention
    expected_file = write_to_temporary_file(expected, name + ".expected.", extention)
    actual_file = write_to_temporary_file(actual, name + ".actual.", extention)
    options.reporter.report(actual_file, expected_file)
    raise AssertionError(
        f'expected != actual\n  actual: "{actual}"\nexpected: "{expected}"'
    )



from abc import abstractmethod, ABC
from typing import Optional


class Namer(ABC):
    APPROVED = ".approved"
    RECEIVED = ".received"

    @abstractmethod
    def get_received_filename(self, base: Optional[str] = None) -> str:
        pass

    @abstractmethod
    def get_approved_filename(self, base: Optional[str] = None) -> str:
        pass



class ReporterMissingException(BaseException):
    def __init__(self, key):
        self.value = key

    def __str__(self):
        return f"Could not find {repr(self.value)} in the environment, perhaps you need to configure your reporter."



from typing import Optional

from approvaltests import Namer


class CliNamer(Namer):
    def __init__(self, test_id: str) -> None:
        self.test_id = test_id

    def get_received_filename(self, base: Optional[str] = None) -> str:
        return f"{self.test_id}.received.txt"

    def get_approved_filename(self, base: Optional[str] = None) -> str:
        return f"{self.test_id}.approved.txt"

    def get_basename(self) -> str:
        return self.test_id



from typing import Optional

from approvaltests.core.namer import Namer
from approvaltests.namer.stack_frame_namer import StackFrameNamer


def get_default_namer(extension: Optional[str] = None) -> Namer:
    return StackFrameNamer(extension)



from threading import local
from typing import Optional
from approvaltests.reporters.diff_reporter import DiffReporter
from approvaltests.core.reporter import Reporter

DEFAULT_REPORTER = local()


def set_default_reporter(reporter: Reporter) -> None:
    DEFAULT_REPORTER.v = reporter


def get_default_reporter() -> Reporter:
    if not hasattr(DEFAULT_REPORTER, "v") or DEFAULT_REPORTER.v is None:
        return DiffReporter()
    return DEFAULT_REPORTER.v


def get_reporter(reporter: Optional[Reporter]) -> Reporter:
    return reporter or get_default_reporter()



from typing import ContextManager, Optional

from approval_utilities.utilities.logger.simple_logger import SimpleLogger
from approvaltests.approvals import verify
from approvaltests.core.options import Options


def verify_simple_logger(
    *,  # enforce keyword arguments - https://www.python.org/dev/peps/pep-3102/
    options: Optional[Options] = None
) -> ContextManager:
    class VerifySimpleLogger:
        def __init__(self):
            self.output = SimpleLogger.log_to_string()

        def __enter__(self):
            pass

        def __exit__(self, exc_type, exc_val, exc_tb):
            verify(self.output, options=options)

    return VerifySimpleLogger()



import itertools
from io import BytesIO
from itertools import product
from typing import Callable, Sequence, Any, Dict, Optional

from mrjob.job import MRJob

from approvaltests import verify, Options


def verify_map_reduce(
    mr_job_under_test: MRJob, test_data: str, *, options: Optional[Options] = None
) -> None:
    storyboard = print_map_reduce_job(mr_job_under_test, test_data)
    verify(storyboard, options=options)


def print_map_reduce_job(mr_job_under_test: MRJob, test_data: str) -> str:
    storyboard = f"{test_data}\n\nMap reduces to:\n\n"
    mr_job_under_test.sandbox(stdin=BytesIO(test_data.encode("utf-8")))
    with mr_job_under_test.make_runner() as runner:
        runner.run()
        results = mr_job_under_test.parse_output(runner.cat_output())
        for key, value in sorted(results):
            storyboard += f"{key}:{value}\n"
    return storyboard


def verify_templated_map_reduce(
    map_reduction: MRJob,
    input_creator: Callable[[Sequence[Any]], str],
    params: Sequence[Any],
    *,
    options: Optional[Options] = None,
) -> None:
    def map_reducer_creator(*_):
        return map_reduction

    verify_templated_map_reduce_with_customized_job(
        map_reducer_creator, input_creator, params, options=options
    )


def verify_templated_map_reduce_with_customized_job(
    map_reduce_creator: Callable[[Sequence[Any]], MRJob],
    input_creator: Callable[[Sequence[Any]], str],
    params: Sequence[Sequence[Any]],
    *,
    options: Optional[Options] = None,
) -> None:
    inputs = product(*params)
    verify_templated_map_reduce_with_customized_job_with_dictionary_args2(
        lambda i: map_reduce_creator(*i),
        lambda i: input_creator(*i),
        inputs,
        options=options,
    )


def product_dict(**kwargs):
    keys = kwargs.keys()
    vals = kwargs.values()
    for instance in itertools.product(*vals):
        yield dict(zip(keys, instance))


def verify_templated_map_reduce_with_customized_job_with_dictionary_args(
    map_reduce_creator: Callable[[Dict[str, Any]], MRJob],
    input_creator: Callable[[Dict[str, Any]], str],
    params: Dict[str, Sequence[Any]],
    *,
    options: Optional[Options] = None,
) -> None:
    inputs = product_dict(**params)
    verify_templated_map_reduce_with_customized_job_with_dictionary_args2(
        map_reduce_creator, input_creator, inputs, options=options
    )


def verify_templated_map_reduce_with_customized_job_with_dictionary_args2(
    map_reduce_creator: Callable[[Any], MRJob],
    input_creator: Callable[[Any], str],
    inputs: Dict[str, Any],
    *,
    options: Optional[Options] = None,
) -> None:
    storyboard = ""
    for input1 in inputs:
        storyboard += f"===================\n\n{input1} =>\n"
        data = input_creator(input1)
        map_reduction = map_reduce_creator(input1)

        storyboard += f"{print_map_reduce_job(map_reduction, data)}\n"
    verify(storyboard, options=options)



from subprocess import call
from typing import List

from approvaltests.core.reporter import Reporter


class ReceivedFileLauncherReporter(Reporter):
    """
    A reporter that attempts to
    open the received file using the
    system default file viewer.

    Depending on the file viewer being launched,
    the test suite execution may halt until the
    user has closed the new process.

    Note: only works on Windows for now.
    """

    @staticmethod
    def get_command(received_path: str) -> List[str]:
        return ["cmd", "/C", "start", received_path, "/B"]

    def report(self, received_path: str, approved_path: str) -> bool:
        command_array = self.get_command(received_path)
        call(command_array)
        return True



import re
from pathlib import Path

from setuptools import setup, find_packages


def get_parent_directory():
    return Path(__file__).parent


def get_version():
    _version_file_contents = (
        get_parent_directory() / "version.py"
    ).read_text()
    matched = re.search(r'"(.*)"', _version_file_contents)
    return matched.group(1) if matched is not None else "UNKNOWN VERSION"


def get_requirements_from_file(file):
    with open(get_parent_directory() / file) as f:
        required = f.read().splitlines()
    return required


def do_the_setup(package_name, package_description, required, extra_requires):
    setup(
        name=package_name,
        version=get_version(),
        description=package_description,
        author="ApprovalTests Contributors",
        author_email="llewellyn.falco@gmail.com",
        url="https://github.com/approvals/ApprovalTests.Python",
        python_requires=">=3.7",
        packages=find_packages(include=["approvaltests*"]),
        package_data={"approvaltests": ["reporters/reporters.json"]},
        install_requires=required,
        extras_require=extra_requires,
        long_description=(get_parent_directory() / "README.md").read_text(),
        long_description_content_type="text/markdown",
        classifiers=[
            "Development Status :: 4 - Beta",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: Apache Software License",
            "Operating System :: POSIX",
            "Operating System :: Microsoft :: Windows",
            "Operating System :: MacOS :: MacOS X",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3 :: Only",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Topic :: Software Development :: Libraries",
            "Topic :: Software Development :: Testing",
            "Topic :: Utilities",
        ],
    )



from approvaltests.core.reporter import Reporter
from approvaltests.reporters.clipboard_reporter import get_command_text
from approval_utilities.utilities.clipboard_utilities import copy_to_clipboard


class ReporterByCopyMoveCommandForEverythingToClipboard(Reporter):
    text = ""

    def report(self, received_path, approved_path):

        ReporterByCopyMoveCommandForEverythingToClipboard.text = (
            ReporterByCopyMoveCommandForEverythingToClipboard.text
            + get_command_text(received_path, approved_path)
            + "\n"
        )
        copy_to_clipboard(ReporterByCopyMoveCommandForEverythingToClipboard.text)
        return True
